"""Handles the GUI elements of the game"""

import os
import cPickle as pickle

import pygame
import pygame.mixer
from pygame.locals import *
from pygame.sprite import Sprite

import sound
from data import *
from util import *
from menu import Menu,universal_keys
import game

class GenericMenu:
    """Generic Menu"""
        
    def __init__(self, screen, parent=None, options=None, title=None, background=None, xalign = GUI_MENU_ALIGN, yalign = GUI_MENU_TOP, color = COLOR_GUI, bgcolor = COLOR_GUI_DARK, hgcolor = COLOR_GUI_HILIGHT):
        """options is a double list of menu strings and functions, which is passed
        to the Menu option."""
        self.screen = screen
        self.parent = parent
        self.title = title
        self.xalign = xalign
        self.yalign = yalign
        self.color = color
        self.bgcolor = bgcolor
        self.hgcolor = hgcolor
        self.options = options        
        self.choice = None
        
        if background != None:
            self.background = load_image(background)
        else:
            self.background = None
        

    def get_choice(self):
        return self.choice

    def on_click(self, choice):        
        if self.options[choice][1]:
            self.options[choice][1]()

    def draw(self):        
        self.menu = Menu(self.screen, map(lambda e: e[0],self.options), self.background, self.title, self.xalign, self.yalign, self.color, self.bgcolor, self.hgcolor)
        self.choice = self.menu.run()            
        self.on_click(self.choice)
        return self.choice


class StartMenu(GenericMenu):
    #Default Variables

    def __init__(self, screen):
        background = "hamster_smash.png"
        title = 'Hamster Smash'
        options = [
            ['Start', self.start_game],
            ['Help', self.help],
            ['Configuration', self.configure],
            ['Scores', self.scores],
            ['Exit', self.quit_game]
        ]
        GenericMenu.__init__(self, screen, None, options, title, background)

    def draw(self):
        done = False
        while not done:
            choice = GenericMenu.draw(self)
            if self.options[choice][0] == 'Exit' or choice==MENU_QUIT:
                done = True
            

    def start_game(self):
        #Start Game
        sound.play_sound("squish")
        score = game.play(self.screen)
        m = ScoreMenu(self.screen)
        if m.is_record(score):
            #They can put an entry on the high score table
            s = ScoreEntry(self.screen,score,"Name: ")
            name = s.draw()
            name = name or 'Anonymous'
            m.insert_record(name, score)
            m.draw()

    def help(self):
        m = HelpMenu(self.screen)
        m.draw()

    def configure(self):
        m = OptionMenu(self.screen)
        m.draw()
        #self.options[self.get_choice()][0] = 'Hi'

    def scores(self):
        m = ScoreMenu(self.screen)
        m.draw()
    
    def quit_game(self):
        pass

class HelpMenu:
    """Help Menu that displays static text and returns on any input."""
    background = "hamster_smash_help.png"
    def __init__(self, screen):
        self.screen = screen
        self.background = load_image(self.background)
        
    def draw(self):
        self.screen.blit(self.background, (0, 0))
        
        ret = render_text("Back",COLOR_GUI_HILIGHT, COLOR_GUI_DARK)
        rect = ret.get_rect()
        rect.bottom = SCREEN_HEIGHT
        rect.right = SCREEN_WIDTH
        self.screen.blit(ret, rect)
        pygame.display.flip()
        
        done = False
        while not done:
            for event in pygame.event.get():
                universal_keys(event,self.screen)
                if event.type == KEYDOWN or event.type == MOUSEBUTTONDOWN:
                    done = True

class OptionMenu(GenericMenu):
    """Help Menu that displays static text and returns on any input."""
    def __init__(self, screen):
        background = "hamster_smash.png"
        title = 'Option Menu'        
        options = [
            ['Music: '+bool_to_str(Variables.vdict["music"]), self.toggle_music],
            ['Sound: '+bool_to_str(Variables.vdict["sound"]), self.toggle_sound],
            ['Fullscreen: '+bool_to_str(Variables.vdict["fullscreen"]), self.toggle_fullscreen],
            #['Redefine Keys', self.define_keys],
            ['Back', None]
        ]
        GenericMenu.__init__(self, screen, None, options, title, background)

    def draw(self):
        done = False
        while not done:
            choice = GenericMenu.draw(self)
            if self.options[choice][0] == 'Back' or choice==MENU_QUIT:
                done = True

        #from variables import Variables
    def toggle_music(self):
        Variables.vdict["music"] = not Variables.vdict["music"]
        self.options[self.get_choice()][0] = 'Music: '+bool_to_str(Variables.vdict["music"])
    
    def toggle_sound(self):
        Variables.vdict["sound"] = not Variables.vdict["sound"]
        self.options[self.get_choice()][0] = 'Sound: '+bool_to_str(Variables.vdict["sound"])

    def toggle_fullscreen(self):
        Variables.vdict["fullscreen"] = not Variables.vdict["fullscreen"]
        self.options[self.get_choice()][0] = 'Fullscreen: '+bool_to_str(Variables.vdict["fullscreen"])
        apply_fullscreen_setting(self.screen)
    
    def define_keys(self):
        pass

class KeyMenu(GenericMenu):
    """Allows Keys to be redefined."""
    def __init__(self, screen):
        pass

#Modified from actual.hermes.gui.py
class ScoreMenu:
    max_pos = 10
    background = "hamster_smash.png"
    def __init__(self, screen):
        self.filename = os.path.join(get_config_path(), 'scores.dat')
        self.screen = screen        
        self.records = []
        try:
            self.records = pickle.load(open(self.filename, 'r'))[:self.max_pos]
        except IOError:
            self.records = [((x+1)*50, 'Thor') for x in xrange(self.max_pos)]
            self.records.sort()
            self.records.reverse()
            
        self.background = load_image(self.background)

    def is_record(self, score):
        return score > min(map(lambda x: x[0], self.records))
        
    def insert_record(self, name, score):
        self.records.append((score, name))
        self.records.sort()
        self.records.reverse()
        self.records.pop()
        fout = file(self.filename, 'w')
        pickle.dump(self.records, fout, -1)
        
    def draw(self):
        self.screen.blit(self.background, (0, 0))
        pygame.display.flip()

        head = render_text("Hamster Smashers:")
        rect = head.get_rect()
        rect.centerx = GUI_MENU_CENTER
        #rect.top = self.yalign + 50 + menu_offset
        score_rect = rect
        self.screen.blit(head, rect)
        
        for x in xrange(0, len(self.records)): 
            score, name = self.records[x]
            
            score_image = render_text(name)
            rect = score_image.get_rect()            
            rect.left = GUI_MENU_CENTER - score_rect.width/2
            rect.top = (60 + rect.height * x )
            self.screen.blit(score_image, rect)
            
            score_image = render_text(str(score))
            rect = score_image.get_rect()            
            rect.right = GUI_MENU_CENTER + score_rect.width/2
            rect.top = (60 + rect.height * x )
            self.screen.blit(score_image, rect)
            
        ret = render_text("Back",COLOR_GUI_HILIGHT, COLOR_GUI_DARK)
        rect = ret.get_rect()
        rect.bottom = SCREEN_HEIGHT
        rect.right = SCREEN_WIDTH
        self.screen.blit(ret, rect)
            
        done = False
        while not done:
            for event in pygame.event.get():
                universal_keys(event,self.screen)
                if event.type == KEYDOWN or event.type == MOUSEBUTTONDOWN:
                    done = True
            pygame.display.flip()

#Modified from actual.hermes.gui.py
class ScoreEntry:
    background = "hamster_smash_score.png"
    def __init__(self, screen, score, prompt):
        self.background = load_image(self.background)
        self.screen = screen
        self.score = score
        self.prompt = prompt
        self.size = 9
        self.text = ''
        
    def handle_event(self, event):
        if event.type == KEYDOWN:
            if event.key in range(256):
                key = chr(event.key)
                if key.isalnum() and len(self.text) < self.size:
                    self.text += key
                    return True
                if event.key == K_BACKSPACE and self.text:
                    self.text = self.text[:-1]
                    return True
                if event.key == K_RETURN:
                    self.done = True
                    return True
        return False
                                          
    def draw(self):        
        self.done = False
        while not self.done:            
            self.screen.blit(self.background, (0, 0))
            
            score = render_text(str(self.score),COLOR_GUI_HILIGHT, COLOR_GUI_DARK)
            rect = score.get_rect()
            rect.centerx = GUI_MENU_CENTER
            rect.top = 60 + rect.height
            score_rect = rect
            self.screen.blit(score, rect)
            
            name = render_text(self.prompt)
            rect = name.get_rect()                
            rect.left = SCREEN_WIDTH/2 - 20
            rect.top = 320
            name_rect = rect
            self.screen.blit(name, rect)
            
            text = render_text(self.text,COLOR_GUI_HILIGHT, COLOR_GUI_DARK)
            rect = text.get_rect()
            rect.left = SCREEN_WIDTH/2 -20 + name_rect.width
            rect.top = 320
            self.screen.blit(text, rect)
            pygame.display.flip()
            
            for event in pygame.event.get():
                self.handle_event(event)

        return self.text
