"""A generic, keyboard- or joystick-controlled menu module.
Draws a list of strings on the screen, and the user can select one of them.

Key configuration is hard-coded:
* Up/down keys or the main vertical axis on the joystick change the selection
* Return, Z, Space or joystick keys 1 or 2 make the selection.
* Esc quits the menu.

To use the module, initialize the Menu classwith the following parameters:
* screen - standard pygame surface the menu is drawn on
* menu_items - a list of strings in the menu

Optional initialization parameters:
* bgscreen - the menu background covering the entire screen, a pygame surface
* heading_text - a heading for the menu

If the bgscreen isn't specified, the class uses a black screen instead.

After initialization, use the run function. If you want some of the items
pre-selected, you can supply it with the index of the selection in the
menu_items list

The run() function returns the index of the selection OR constant MENU_QUIT,
if the user presses Esc while in the menu.

Menu placement on the screen is controlled by constants in the locals.py.
The use of the module also requires play_sound from the sound module,
Util class and render_text from the util module
and the Variables class from the variables module."""

import pygame
import os
import sys

from pygame.locals import *

from locals import *

import data

from util import Util, render_text
from variables import Variables

from sound import play_sound, stop_music

def universal_keys(e,screen):
    """Accept some universal OS keypresses, such as Ctrl-C
    and Alt-F4 for exiting, Alt+Enter for fullscreen, etc."""
    if e.type == pygame.QUIT: 
        sys.exit()
    elif e.type is KEYDOWN:
        if ((e.key == K_F4
                and (e.mod&(KMOD_LALT|KMOD_RALT)) != 0
                and (e.mod&(KMOD_LCTRL|KMOD_RCTRL)) == 0) or
            (e.key == K_c
                and (e.mod&(KMOD_LALT|KMOD_RALT)) == 0
                and (e.mod&(KMOD_LCTRL|KMOD_RCTRL)) != 0)):
            sys.exit(0)
        elif (e.key == K_BACKSPACE):
            pygame.image.save(screen, GAME_NAME_SHORT+'.bmp')
        elif ((e.key == K_RETURN
                and (e.mod&(KMOD_LALT|KMOD_RALT)) != 0
                and (e.mod&(KMOD_LCTRL|KMOD_RCTRL)) == 0)): 
            apply_fullscreen_setting(screen)
        #Any toggle status keys here
        elif (e.key == K_m):
            Variables.vdict["music"] = not Variables.vdict["music"]
            if not Variables.vdict["music"]:
                stop_music()
        elif (e.key == K_s):
            Variables.vdict["sound"] = not Variables.vdict["sound"]


class Menu:

    def __init__(self, screen, menu_items, bgscreen = None,
    heading_text = None, xalign = GUI_MENU_ALIGN, yalign = GUI_MENU_TOP, color = COLOR_GUI, bgcolor = COLOR_GUI_DARK, hgcolor = COLOR_GUI_HILIGHT):     
        """Init the menu. Screen is a Pygame Surface, menu_items is a list of
        strings in the menu, bgscreen is the menu background Surface and
        heading_text is a string for the heading."""
        self.bgscreen = bgscreen
        if self.bgscreen == None:
            self.bgscreen = Util.blackscreen
        self.screen = screen
        self.menu_items = menu_items
        self.heading_text = heading_text
        
        self.xalign = xalign
        self.yalign = yalign
        self.color = color
        self.bgcolor = bgcolor
        self.hgcolor = hgcolor
        return

    def run(self, menu_choice = 0):
        """Run the menu and return the user's choice."""
        
        done = False

        clock = pygame.time.Clock()

        #Renders the menu background, usually the faded out game display
        #Or a black screen
        self.screen.blit(self.bgscreen, (0, 0))

        #Menu loop

        while not done:

            # Pygame event and keyboard input processing
            for event in pygame.event.get():
                universal_keys(event,self.screen)
                
                if event.type == QUIT or \
                (event.type == KEYDOWN and event.key == K_ESCAPE) or \
                (GUI_MOUSE and 
                (event.type == MOUSEBUTTONDOWN and event.button == 3)) or \
                (event.type == JOYBUTTONDOWN and \
                (event.button == 1)):
                    menu_choice = MENU_QUIT
                    done = True
                elif (event.type == KEYDOWN and event.key == K_DOWN) or \
                (GUI_MOUSE and
                (event.type == MOUSEBUTTONDOWN and event.button == 5)) or \
                (event.type == JOYAXISMOTION and event.axis == 1 and \
                event.value > 0.7):
                    if menu_choice + 1 < len(self.menu_items):
                        menu_choice += 1
                        play_sound("click")
                    elif GUI_WRAP:
                        menu_choice = 0
                        play_sound("click")
                elif (event.type == KEYDOWN and event.key == K_UP) or \
                (GUI_MOUSE and
                (event.type == MOUSEBUTTONDOWN and event.button == 4)) or \
                (event.type == JOYAXISMOTION and event.axis == 1 and \
                event.value < -0.7):
                    if menu_choice > 0:
                        menu_choice -= 1
                        play_sound("click")
                    elif GUI_WRAP:
                        menu_choice = len(self.menu_items) - 1
                        play_sound("click")
                elif (event.type == KEYDOWN and \
                (event.key == K_z or event.key == K_SPACE or \
                event.key == K_RETURN)) or \
                (GUI_MOUSE and
                (event.type == MOUSEBUTTONDOWN and event.button == 1)) or \
                (event.type == JOYBUTTONDOWN and \
                (event.button == 0)):
                    done = True

            #Menu rendering

            #Menu offset value centers the menu when the maximum amount of
            #choices is not visible

            if len(self.menu_items) < MENU_MAX_VISIBLE:
                menu_offset = -(len(self.menu_items) - 5) * 10
            else:
                menu_offset = -(MENU_MAX_VISIBLE - 5) * 10
                    
            #If you want a transparent extra background picture for the menu,
            #rendering it would go here

            if self.heading_text != None:
                #We need another render_text method here******************
                menu_head = render_text(self.heading_text)
                rect = menu_head.get_rect()
                rect.centerx = GUI_MENU_CENTER
                rect.top = self.yalign + 50 + menu_offset
                heading_rect = rect
                self.screen.blit(menu_head, rect)
            
            #If the menu choice is greater than the second last menu item
            #on screen, the menu must be scrolled:
            if menu_choice > (MENU_MAX_VISIBLE - 1):
                current_menu_index = menu_choice - MENU_MAX_VISIBLE
                if (menu_choice + 1) < len(self.menu_items):
                    current_menu_index += 1
            else:
                current_menu_index = 0

            menu_visible = 0

            while (not (menu_visible > MENU_MAX_VISIBLE or
            (current_menu_index) == len(self.menu_items))):
                m = self.menu_items[current_menu_index]
                if (menu_choice == current_menu_index):
                    menu_image = render_text(m, self.hgcolor,
                    self.bgcolor)
                else:
                    menu_image = render_text(m, self.color)
                rect = menu_image.get_rect()
                if self.xalign == MENU_ALIGN_LEFT:
                    rect.left = GUI_MENU_CENTER - heading_rect.width/2
                elif self.xalign == MENU_ALIGN_RIGHT:
                    rect.right = GUI_MENU_CENTER + heading_rect.width/2
                else: #self.xalign == MENU_ALIGN_CENTER:
                    rect.centerx = GUI_MENU_CENTER
                #rect.top = (GUI_MENU_TOP + 60 + (menu_visible + 1) * 20 + menu_offset)
                rect.top = (GUI_MENU_TOP + 60 + (menu_visible + 1) * rect.height + menu_offset)

                self.screen.blit(menu_image, rect)
                
                if GUI_MOUSE:
                    x,y = pygame.mouse.get_pos()
                    mouserect = Rect(x,y,1,1)
                    if rect.contains(mouserect) and menu_choice != current_menu_index:
                        menu_choice = current_menu_index
                        play_sound("click")   
                        

                current_menu_index += 1
                menu_visible += 1



            #Display, clock

            pygame.display.flip()

            clock.tick(FPS)

        return menu_choice