""" This module contains generic utility functions.
The comments give a basic idea of each function's purpose,
and they can also be found from the WWW documentation."""

import pygame
from pygame.locals import *
from string import lower
import os
import codecs
import datetime

from locals import *

import data

from variables import Variables

from log import error_message, log_message

class Util:
    """A storage class for cached images, variables used by the
    utility functions and such."""
    pygame.font.init()
    smallfont = pygame.font.Font(data.filepath(\
    os.path.join("misc", FONT_NAME)), FONT_SIZE)
    countdownfont = pygame.font.Font(data.filepath(\
    os.path.join("misc", FONT_NAME)), 30)
    cached_text_images = {}
    fade_state = FADE_STATE_BLACK
    blackscreen = pygame.Surface((SCREEN_WIDTH, SCREEN_HEIGHT))

def get_config_path():
    """Return a path for saving config in the user's home directory.
    It's compatible with both UNIX-likes (environment variable HOME)
    and Windows (environment variable APPDATA).
    Prints error messages if unsuccesful, and reverts to a "saves" directory
    under the data directory instead.
    BE SURE TO CHANGE THE GAME_NAME_SHORT CONSTANT BY EDITING locals.py
    BEFORE USING THIS FUNCTION!"""    
    path_name = ""
    try:
        path_name = os.path.join(os.environ["HOME"], "." + GAME_NAME_SHORT)
    except:
        error_message("Couldn't find environment variable HOME, " +
        "reverting to APPDATA. This is normal under Windows.")
        try:
            path_name = os.path.join(os.environ["APPDATA"], GAME_NAME_SHORT)
        except:
            error_message("Couldn't get environment variable for home " +
            "directory, using data directory instead.")
            path_name = data.filepath("saves")
    if not os.path.exists(path_name):
        os.mkdir(path_name)
    return path_name

def parse_config():
    """Parse a config file stored in the location given by get_config_path().
    The "sound" and "fullscreen" values are parsed by default, but you can 
    add your own by editing the code. The parsed values are stored in the 
    Variables class of the variables module."""
    #Default values for configuration (example: sound is on):
    Variables.vdict["music"] = True
    Variables.vdict["sound"] = True
    Variables.vdict["fullscreen"] = False 

    #Setting this to True will enable printing error messages to the console.
    #Might be useful during debugging.
    #You may set it to False in a release version to disable console output.
    #Messages will still be added to the message log file.
    Variables.vdict["verbose"] = False

    file_path = os.path.join(get_config_path(), "config.txt")
    try:
        conffile = codecs.open(file_path, "r", "utf_8")
        for line in conffile.xreadlines():
            if is_config_line(line):
                values = line.split("\t")

                #Parsing configuration values:

                if values[0] == "sound" or values[0] == 'music':
                    Variables.vdict[values[0]] = str_to_bool(values[1])
                    #The "sound" and "fullscreen" values are of type bool

                #Add your own value types here.
                #if values[0] == "fullscreen":
                #    Variables.vdict[values[0]] = str_to_bool(values[1])
                    

    except:
        if write_config():
            log_message("Created configuration file to " + file_path)

    return

def write_config():
    """Write the config stored in the Variables class to the configuration
    file. Prints an error message if unsuccessful."""
    file_path = os.path.join(get_config_path(), "config.txt")
    try:
        conffile = codecs.open(file_path, "w", "utf_8")

        #Variables to be saved:
        print >> conffile, "sound\t%s" % bool_to_str(Variables.vdict["sound"])
        print >> conffile, "music\t%s" % bool_to_str(Variables.vdict["music"])
        #print >> conffile, "fullscreen\t%s" % \
        #bool_to_str(Variables.vdict["fullscreen"])

    except:
        error_message("Could not write configuration file to " + file_path)
        return False

    return True

def write_log():
    """Write the message log to disk."""
    file_path = os.path.join(get_config_path(), "log.txt")
    old_log = ""
    if os.path.exists(file_path):
        conffile = codecs.open(file_path, "r", "utf_8")
        count = 0
        for line in conffile.xreadlines():
            old_log = old_log + line
            count += 1
            if count > MAX_OLD_LOG_LINES:
                break
    if Variables.vdict.has_key("log"):
        try:
            conffile = codecs.open(file_path, "w", "utf_8")
            print >> conffile, "Log updated " + str(datetime.date.today())
            print >> conffile, Variables.vdict["log"]
            print >> conffile, ""
            print >> conffile, old_log
        except:
            error_message("Could not write log file to " + file_path)
            return False
    return True

def is_config_line(line):
    """Return true if the line is not empty or a comment starting with #"""
    if line.strip() == "":
        return False
    if (line.strip()).startswith("#"):
        return False
    return True

def str_to_bool(string):
    """Convert string to bool"""
    formatted = string.strip()
    formatted = lower(formatted)
    return (formatted == "true" or formatted == "1" or formatted == "on")

def bool_to_str(bool):
    """Convert bool to string"""
    if bool:
        return "on"
    else:
        return "off"

def render_text(string, color = COLOR_GUI, bgcolor = COLOR_GUI_BG):
    """Render fancy-looking text. The function also handles caching of text
    images. Returns a pygame surface containing the rendered text.
    The text is rendered with slight edges to make it look more readable on a
    colorful background.

    The constant colors can be found from locals.py."""
    if Util.cached_text_images.has_key(string + str(color) + str(bgcolor)):
        final_image = Util.cached_text_images[string + str(color) +
        str(bgcolor)]
    else:
        text_image_bg = Util.smallfont.render(string, True, bgcolor)
        text_image_fg = Util.smallfont.render(string, True, color)
        rect = text_image_bg.get_rect()
        final_image = pygame.Surface((rect.width + 2, rect.height + 2))\
        .convert_alpha()

        final_image.fill((0,0,0,0))

        final_image.blit(text_image_bg, rect)
        final_image.blit(text_image_bg, (1,0))
        final_image.blit(text_image_bg, (2,0))

        final_image.blit(text_image_bg, (0,2))
        final_image.blit(text_image_bg, (1,2))
        final_image.blit(text_image_bg, (2,2))

        final_image.blit(text_image_fg, (1,1))
        Util.cached_text_images[string + str(color) + str(bgcolor)]\
        = final_image
    return final_image

def render_text_dialogue(screen, string, phase):
    """Render text partially.
    For fancy dialogue display.
    The phase value is the amount of characters shown.
    -1 phase means that the whole string is visible.
    """
    if phase == -1:
        phase = len(string)

    rendered_string = string[0:phase]
    string_image = render_text(rendered_string)
    rect = string_image.get_rect()
    rect.centerx = SCREEN_WIDTH / 2
    rect.centery = SCREEN_HEIGHT / 2
    screen.blit(string_image, rect)

    if phase < len(string):
        phase += 1
        #You can add a sound effect here, if you want, example code:
        #play_sound("click")
    else:
        return -1

    return phase

def fade_to_black(screen, fade_target):
    """Fades the screen to black. Both fade-in and fade-out.
    Returns true if the fading has finished.
    fade_target should be an integer (0-255)
    * 255 = FADE_STATE_BLACK : The display is all black
    * 0 = FADE_STATE_NONE : The display is not faded at all"""
    if Util.fade_state > fade_target:
        Util.fade_state += int(255 / (FPS * FADE_IN))
        if Util.fade_state < fade_target:
            Util.fade_state = fade_target
    if Util.fade_state < fade_target:
        Util.fade_state -= -int(255 / (FPS * FADE_OUT))
        if Util.fade_state > fade_target:
            Util.fade_state = fade_target
    if Util.fade_state > FADE_STATE_NONE:
        Util.blackscreen.set_alpha(Util.fade_state)
        screen.blit(Util.blackscreen, screen.get_rect())
    return (Util.fade_state == fade_target)

def apply_fullscreen_setting(screen):
    """Apply the fullscreen setting stored in variables to the screen.
    screen is a pygame standard surface.
    Contents of the screen should not be affected."""
    mode = 0
    if Variables.vdict["fullscreen"]:
        mode ^= FULLSCREEN
    tmp = pygame.Surface((SCREEN_WIDTH,SCREEN_HEIGHT)).convert()
    tmp.blit(screen,(0,0))
    screen = pygame.display.set_mode((SCREEN_WIDTH,SCREEN_HEIGHT),mode)
    screen.blit(tmp,(0,0))
    pygame.display.flip()
    return
